package cn.ibizlab.core.uaa.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.core.uaa.domain.SYS_PERMISSION;
import cn.ibizlab.core.uaa.filter.SYS_PERMISSIONSearchContext;
import cn.ibizlab.core.uaa.service.ISYS_PERMISSIONService;

import cn.ibizlab.util.helper.CachedBeanCopier;



import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.core.uaa.mapper.SYS_PERMISSIONMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;


/**
 * 实体[权限/资源] 服务对象接口实现
 */
@Slf4j
@Service("SYS_PERMISSIONServiceImpl")
public class SYS_PERMISSIONServiceImpl extends ServiceImpl<SYS_PERMISSIONMapper, SYS_PERMISSION> implements ISYS_PERMISSIONService {

    @Autowired
    @Lazy
    private cn.ibizlab.core.uaa.service.ISYS_ROLE_PERMISSIONService sysRolePermissionService;
    private int batchSize = 500;

    @Override
    @Transactional
    public boolean save(SYS_PERMISSION et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(SYS_PERMISSION et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public void saveBatch(List<SYS_PERMISSION> list) {
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    public SYS_PERMISSION getDraft(SYS_PERMISSION et) {
        return et;
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        sysRolePermissionService.removeByPermissionid(key) ;
        if(!removeById(key)) return false;
        return true ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public boolean update(SYS_PERMISSION et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("sys_permissionid",et.getPermissionid())))
            return false;
        sysRolePermissionService.saveByPermissionid(et.getPermissionid(),et.getSysRolePermission());
        CachedBeanCopier.copy(get(et.getPermissionid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<SYS_PERMISSION> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean create(SYS_PERMISSION et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        sysRolePermissionService.saveByPermissionid(et.getPermissionid(),et.getSysRolePermission());
        CachedBeanCopier.copy(get(et.getPermissionid()),et);
        return true;
    }

    @Override
    public void createBatch(List<SYS_PERMISSION> list) {
        saveBatch(list,batchSize);
    }

    @Override
    public boolean checkKey(SYS_PERMISSION et) {
        return (!ObjectUtils.isEmpty(et.getPermissionid()))&&(!Objects.isNull(this.getById(et.getPermissionid())));
    }

    @Override
    @Transactional
    public SYS_PERMISSION get(String key) {
        SYS_PERMISSION et = getById(key);
        if(et==null){
            et=new SYS_PERMISSION();
            et.setPermissionid(key);
        }
        else{
            et.setSysRolePermission(sysRolePermissionService.selectByPermissionid(key));
        }
        return et;
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<SYS_PERMISSION> searchDefault(SYS_PERMISSIONSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<SYS_PERMISSION> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<SYS_PERMISSION>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }


    @Override
    public List<JSONObject> select(String sql){
        return this.baseMapper.selectBySQL(sql);
    }

    @Override
    @Transactional
    public boolean execute(String sql){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

}


