package cn.ibizlab.core.uaa.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.core.uaa.domain.SYS_ROLE;
import cn.ibizlab.core.uaa.filter.SYS_ROLESearchContext;
import cn.ibizlab.core.uaa.service.ISYS_ROLEService;

import cn.ibizlab.util.helper.CachedBeanCopier;



import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.core.uaa.mapper.SYS_ROLEMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;


/**
 * 实体[系统角色] 服务对象接口实现
 */
@Slf4j
@Service("SYS_ROLEServiceImpl")
public class SYS_ROLEServiceImpl extends ServiceImpl<SYS_ROLEMapper, SYS_ROLE> implements ISYS_ROLEService {

    @Autowired
    @Lazy
    private cn.ibizlab.core.uaa.service.ISYS_ROLE_PERMISSIONService sysRolePermissionService;
    @Autowired
    @Lazy
    private cn.ibizlab.core.uaa.service.ISYS_USER_ROLEService sysUserRoleService;
    private int batchSize = 500;

    @Override
    @Transactional
    public boolean save(SYS_ROLE et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(SYS_ROLE et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public void saveBatch(List<SYS_ROLE> list) {
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(SYS_ROLE et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("sys_roleid",et.getRoleid())))
            return false;
        sysRolePermissionService.saveByRoleid(et.getRoleid(),et.getSysRolePermission());
        sysUserRoleService.saveByRoleid(et.getRoleid(),et.getSysUserRole());
        CachedBeanCopier.copy(get(et.getRoleid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<SYS_ROLE> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean create(SYS_ROLE et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        sysRolePermissionService.saveByRoleid(et.getRoleid(),et.getSysRolePermission());
        sysUserRoleService.saveByRoleid(et.getRoleid(),et.getSysUserRole());
        CachedBeanCopier.copy(get(et.getRoleid()),et);
        return true;
    }

    @Override
    public void createBatch(List<SYS_ROLE> list) {
        saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public SYS_ROLE get(String key) {
        SYS_ROLE et = getById(key);
        if(et==null){
            et=new SYS_ROLE();
            et.setRoleid(key);
        }
        else{
            et.setSysRolePermission(sysRolePermissionService.selectByRoleid(key));
            et.setSysUserRole(sysUserRoleService.selectByRoleid(key));
        }
        return et;
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        sysRolePermissionService.removeByRoleid(key) ;
        sysUserRoleService.removeByRoleid(key) ;
        if(!removeById(key)) return false;
        return true ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    public SYS_ROLE getDraft(SYS_ROLE et) {
        return et;
    }

    @Override
    public boolean checkKey(SYS_ROLE et) {
        return (!ObjectUtils.isEmpty(et.getRoleid()))&&(!Objects.isNull(this.getById(et.getRoleid())));
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<SYS_ROLE> searchDefault(SYS_ROLESearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<SYS_ROLE> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<SYS_ROLE>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }


    @Override
    public List<JSONObject> select(String sql){
        return this.baseMapper.selectBySQL(sql);
    }

    @Override
    @Transactional
    public boolean execute(String sql){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

}


