package cn.ibizlab.core.uaa.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.core.uaa.domain.SYS_USER;
import cn.ibizlab.core.uaa.filter.SYS_USERSearchContext;
import cn.ibizlab.core.uaa.service.ISYS_USERService;

import cn.ibizlab.util.helper.CachedBeanCopier;



import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.core.uaa.mapper.SYS_USERMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;


/**
 * 实体[系统用户] 服务对象接口实现
 */
@Slf4j
@Service("SYS_USERServiceImpl")
public class SYS_USERServiceImpl extends ServiceImpl<SYS_USERMapper, SYS_USER> implements ISYS_USERService {

    @Autowired
    @Lazy
    private cn.ibizlab.core.uaa.service.ISYS_USER_ROLEService sysUserRoleService;
    private int batchSize = 500;

    @Override
    @Transactional
    public boolean remove(String key) {
        sysUserRoleService.removeByUserid(key) ;
        if(!removeById(key)) return false;
        return true ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public boolean update(SYS_USER et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("userid",et.getUserid())))
            return false;
        sysUserRoleService.saveByUserid(et.getUserid(),et.getSysUserRole());
        CachedBeanCopier.copy(get(et.getUserid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<SYS_USER> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    public boolean checkKey(SYS_USER et) {
        return (!ObjectUtils.isEmpty(et.getUserid()))&&(!Objects.isNull(this.getById(et.getUserid())));
    }

    @Override
    @Transactional
    public SYS_USER get(String key) {
        SYS_USER et = getById(key);
        if(et==null){
            et=new SYS_USER();
            et.setUserid(key);
        }
        else{
            et.setSysUserRole(sysUserRoleService.selectByUserid(key));
        }
        return et;
    }

    @Override
    @Transactional
    public boolean save(SYS_USER et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(SYS_USER et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public void saveBatch(List<SYS_USER> list) {
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    public SYS_USER getDraft(SYS_USER et) {
        return et;
    }

    @Override
    @Transactional
    public boolean create(SYS_USER et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        sysUserRoleService.saveByUserid(et.getUserid(),et.getSysUserRole());
        CachedBeanCopier.copy(get(et.getUserid()),et);
        return true;
    }

    @Override
    public void createBatch(List<SYS_USER> list) {
        saveBatch(list,batchSize);
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<SYS_USER> searchDefault(SYS_USERSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<SYS_USER> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<SYS_USER>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }


    @Override
    public List<JSONObject> select(String sql){
        return this.baseMapper.selectBySQL(sql);
    }

    @Override
    @Transactional
    public boolean execute(String sql){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

}


