package cn.ibizlab.core.uaa.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.core.uaa.domain.SYS_USER_ROLE;
import cn.ibizlab.core.uaa.filter.SYS_USER_ROLESearchContext;
import cn.ibizlab.core.uaa.service.ISYS_USER_ROLEService;

import cn.ibizlab.util.helper.CachedBeanCopier;



import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.core.uaa.mapper.SYS_USER_ROLEMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;


/**
 * 实体[用户角色关系] 服务对象接口实现
 */
@Slf4j
@Service("SYS_USER_ROLEServiceImpl")
public class SYS_USER_ROLEServiceImpl extends ServiceImpl<SYS_USER_ROLEMapper, SYS_USER_ROLE> implements ISYS_USER_ROLEService {

    private int batchSize = 500;

    @Override
    public boolean checkKey(SYS_USER_ROLE et) {
        return (!ObjectUtils.isEmpty(et.getUserroleid()))&&(!Objects.isNull(this.getById(et.getUserroleid())));
    }

    @Override
    public SYS_USER_ROLE getDraft(SYS_USER_ROLE et) {
        return et;
    }

    @Override
    @Transactional
    public boolean update(SYS_USER_ROLE et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("sys_user_roleid",et.getUserroleid())))
            return false;
        CachedBeanCopier.copy(get(et.getUserroleid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<SYS_USER_ROLE> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        if(!removeById(key)) return false;
        return true ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public boolean create(SYS_USER_ROLE et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getUserroleid()),et);
        return true;
    }

    @Override
    public void createBatch(List<SYS_USER_ROLE> list) {
        saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public SYS_USER_ROLE get(String key) {
        SYS_USER_ROLE et = getById(key);
        if(et==null){
            et=new SYS_USER_ROLE();
            et.setUserroleid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public boolean save(SYS_USER_ROLE et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(SYS_USER_ROLE et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public void saveBatch(List<SYS_USER_ROLE> list) {
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<SYS_USER_ROLE> selectByRoleid(String roleid) {
        return baseMapper.selectByRoleid(roleid);
    }

    @Override
    public void removeByRoleid(String roleid) {
        this.remove(new QueryWrapper<SYS_USER_ROLE>().eq("sys_roleid",roleid));
    }

	@Override
    public void saveByRoleid(String roleid,List<SYS_USER_ROLE> list) {
        if(list==null)
            return;
        Set<String> delIds=new HashSet<String>();
        List<SYS_USER_ROLE> _update=new ArrayList<SYS_USER_ROLE>();
        List<SYS_USER_ROLE> _create=new ArrayList<SYS_USER_ROLE>();
        for(SYS_USER_ROLE before:selectByRoleid(roleid)){
            delIds.add(before.getUserroleid());
        }
        for(SYS_USER_ROLE sub:list) {
            sub.setRoleid(roleid);
            if(ObjectUtils.isEmpty(sub.getUserroleid()))
                sub.setUserroleid((String)sub.getDefaultKey(true));
            if(delIds.contains(sub.getUserroleid())) {
                delIds.remove(sub.getUserroleid());
                _update.add(sub);
            }
            else
                _create.add(sub);
        }
        if(_update.size()>0)
            this.updateBatch(_update);
        if(_create.size()>0)
            this.createBatch(_create);
        if(delIds.size()>0)
            this.removeBatch(delIds);
	}

	@Override
    public List<SYS_USER_ROLE> selectByUserid(String userid) {
        return baseMapper.selectByUserid(userid);
    }

    @Override
    public void removeByUserid(String userid) {
        this.remove(new QueryWrapper<SYS_USER_ROLE>().eq("sys_userid",userid));
    }

	@Override
    public void saveByUserid(String userid,List<SYS_USER_ROLE> list) {
        if(list==null)
            return;
        Set<String> delIds=new HashSet<String>();
        List<SYS_USER_ROLE> _update=new ArrayList<SYS_USER_ROLE>();
        List<SYS_USER_ROLE> _create=new ArrayList<SYS_USER_ROLE>();
        for(SYS_USER_ROLE before:selectByUserid(userid)){
            delIds.add(before.getUserroleid());
        }
        for(SYS_USER_ROLE sub:list) {
            sub.setUserid(userid);
            if(ObjectUtils.isEmpty(sub.getUserroleid()))
                sub.setUserroleid((String)sub.getDefaultKey(true));
            if(delIds.contains(sub.getUserroleid())) {
                delIds.remove(sub.getUserroleid());
                _update.add(sub);
            }
            else
                _create.add(sub);
        }
        if(_update.size()>0)
            this.updateBatch(_update);
        if(_create.size()>0)
            this.createBatch(_create);
        if(delIds.size()>0)
            this.removeBatch(delIds);
	}


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<SYS_USER_ROLE> searchDefault(SYS_USER_ROLESearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<SYS_USER_ROLE> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<SYS_USER_ROLE>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }


    @Override
    public List<JSONObject> select(String sql){
        return this.baseMapper.selectBySQL(sql);
    }

    @Override
    @Transactional
    public boolean execute(String sql){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

}


